﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Encodings.BinaryVectorEncoding;

namespace HeuristicLab.Problems.NK {
    
  [Item("Lexicographic Binary Vector Comparer", "Compares two binary vectors lexicographically")]
  [StorableClass]
  public sealed class LexicographicBinaryVectorComparer : BinaryVectorComparer {

    [StorableConstructor]
    protected LexicographicBinaryVectorComparer(bool deserializing) : base(deserializing) { }
    protected LexicographicBinaryVectorComparer(LexicographicBinaryVectorComparer original, Cloner cloner)
      : base(original, cloner) {
    }
    public LexicographicBinaryVectorComparer() { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new LexicographicBinaryVectorComparer(this, cloner);
    }

    public override int Compare(BinaryVector x, BinaryVector y) {
      for (int i = 0; i<Math.Min(x.Length, y.Length); i++) {
        if (!x[i] && y[i])
          return -1;
        if (x[i] && !y[i])
          return 1;
      }
      if (x.Length > y.Length)
        return 1;
      if (x.Length < y.Length)
        return -1;
      return 0;
    }
  }
}
